@@ -1,6 +1,6 @@
 ;;; ebdb-i18n.el --- Internationalization support for EBDB  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2016-2017  Free Software Foundation, Inc.
+;; Copyright (C) 2016-2018  Free Software Foundation, Inc.
 
 ;; Author: Eric Abrahamsen <eric@ericabrahamsen.net>
 
@@ -681,7 +681,7 @@ for their symbol representations."
 				       &optional slots obj)
   (let ((country
 	  (cdr (assoc (completing-read
-		       "Country: "
+		       "Address country: "
 		       (ebdb-i18n-countries)
 		       nil nil
 		       (when obj (car (rassoc (ebdb-address-country obj)
@@ -736,7 +736,7 @@ for their symbol representations."
 	 (if obj
 	     (slot-value obj 'country-code)
 	   (cdr (assoc (completing-read
-			"Country/Region: "
+			"Phone number country/region: "
 			ebdb-i18n-phone-codes nil nil)
 		       ebdb-i18n-phone-codes))))
 	(area-code (when obj (slot-value obj 'area-code))))
@@ -823,7 +823,7 @@ for their symbol representations."
 	(cl-no-method nil)))
     str))
 
-(cl-defmethod ebdb-init-field :extra "i18n" ((name ebdb-field-name) &optional record)
+(cl-defmethod ebdb-init-field :extra "i18n" ((name ebdb-field-name) record)
   "Do additional initialization work for international names."
   (let* ((res (cl-call-next-method name record))
 	 (str (ebdb-string name))
@@ -834,7 +834,7 @@ for their symbol representations."
 	(cl-no-method nil)))
     res))
 
-(cl-defmethod ebdb-delete-field :extra "i18n" ((name ebdb-field-name) &optional record unload)
+(cl-defmethod ebdb-delete-field :extra "i18n" ((name ebdb-field-name) record &optional unload)
   "Do additional deletion work for international names."
   (let* ((str (ebdb-string name))
 	 (script (aref char-script-table (aref str 0))))
