@@ -1,6 +1,6 @@
 ;;; ebdb-snarf.el --- Creating or displaying records based on free-form pieces of text  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2016-2017  Free Software Foundation, Inc.
+;; Copyright (C) 2016-2018  Free Software Foundation, Inc.
 
 ;; Author: Eric Abrahamsen <eric@ericabrahamsen.net>
 ;; Keywords: mail
@@ -46,14 +46,13 @@
 (require 'ebdb-com)
 
 (defcustom ebdb-snarf-routines
-  `((ebdb-field-mail "[[:blank:]([<\"]*\\([^[:space:]\":\n<[]+@[^]:[:space:])>\"\n]+\\)")
+  `((ebdb-field-mail "[[:blank:]([<\"]*\\([[:alnum:]][^[:space:]\":\n<[]+@[^]:[:space:])>\"\n]+[[:alnum:]]\\)")
     (ebdb-field-url ,(concat "\\("
 			     (regexp-opt ebdb-url-valid-schemes)
 			     "//[^ \n\t]+\\)"))
-    (ebdb-field-phone "\\(\\+?[[:digit:]]\\{1,3\\}[ )-.]?[[:digit:] -.()]+\\)"))
+    (ebdb-field-phone "\\(\\+?[[:digit:]]\\{1,3\\}[ )-.]?[[:digit:] -.()]\\{6,\\}\\)"))
 
   "An alist of EBDB field classes and related regexps.
-
 Each alist element is an EBDB field class symbol, followed by a
 list of regular expressions that can be used to produce instances
 of that class when passed to `ebdb-parse'.  Each regular
@@ -65,10 +64,9 @@ expression should contain at least one parenthetical group: the
   :type 'list)
 
 (defcustom ebdb-snarf-name-re
-  (list "\\(?:[[:upper:]][[:lower:]'-]+[,.[:space:]]*\\)\\{2,\\}")
+  (list "\\(?:[[:upper:]][[:lower:]'-]+[,.[:blank:]]*\\)\\{2,\\}")
 
   "A list of regular expressions matching names.
-
 This is a separate option from `ebdb-snarf-routines' because
 snarfing doesn't search for names separately, only in conjunction
 with other field types.
@@ -82,7 +80,6 @@ groups."
 ;;;###autoload
 (defun ebdb-snarf (&optional string start end recs ret)
   "Snarf text and attempt to display/update/create a record from it.
-
 If STRING is given, snarf the string.  If START and END are given
 in addition to STRING, assume they are 0-based indices into it.
 If STRING is nil but START and END are given, assume they are
@@ -119,15 +116,15 @@ If RET is non-nil, return the records.  Otherwise display them."
 
 (defun ebdb-snarf-collect (str &optional records)
   "Collect EBDB record information from string STR.
-
 This function will find everything that looks like field
 information, and do its best to organize it into likely groups.
 If RECORDS is given, it should be a list of records that we think
-have something to do with the text in the buffer.
+have something to do with the text in the string.
 
 This function returns a list of vectors.  Each vector contains
 three elements: a record, a list of name-class instances, and a
-list of other field instances.  Any element can be nil."
+list of other field instances.  Any of the three elements can be
+nil."
   (let ((case-fold-search nil)
 	;; BUNDLES is the list of vectors.  If RECORDS is given, then
 	;; we have something to start with.
@@ -149,17 +146,15 @@ list of other field instances.  Any element can be nil."
 
 	;; (Also see the snarfing tests in ebdb-test.el.)
 
-	;; The tactic is: Make a big regexp that finds any probable
-	;; field data.  Once there's a hit, search *backwards* for a
-	;; name, and *forwards* for more fields.  All contiguous field
-	;; data is grouped into the same bundle.  If the first field
-	;; found is at bol, assume "block" style data, as in the third
-	;; example above.  If it is not at bol, assume "inline" style
-	;; data, as in the second example.
+	;; The tactic is: Make a big regexp that finds a big blob of
+	;; probable field data.  Once there's a hit, search
+	;; *backwards* for a name, and *forwards* for more fields.
+	;; All contiguous field data is grouped into the same bundle.
 
 	;; Snarfing mail message data is very common, it would be nice
 	;; to somehow disregard left-hand quotation characters and
-	;; indendation.  A problem for another day.
+	;; indentation.  See `mail-citation-prefix-regexp'.  A problem
+	;; for another day.
 	(big-re
 	 (concat
 	  "\\(?:"
@@ -169,78 +164,83 @@ list of other field instances.  Any element can be nil."
 		 (cadr r)
 	       (mapconcat #'identity (cadr r) "\\|")))
 	   ebdb-snarf-routines
-	   "\\|")
-	  "\\)"))
-	bundle block name)
+	   "\\|*")
+	  "\\)+"))
+	(name-re (concat
+		  "\\("
+		  (mapconcat #'identity
+			     ebdb-snarf-name-re "\\|")
+		  "\\)[-\n ,:]*"))
+	field seen-fields)
 
     (with-temp-buffer
-      (insert str)
+      ;; Snarfing mail buffers is very common, try deleting citation
+      ;; prefixes from the buffer first.
+      (insert (replace-regexp-in-string
+	       (concat "^" mail-citation-prefix-regexp "[:blank:]+")
+	       "" str))
       (goto-char (point-min))
+      ;; SOMETHING from the big-re matched.
       (while (re-search-forward big-re nil t)
 	(goto-char (match-beginning 0))
-	(setq block (= (point) (point-at-bol)))
-	(when (setq name
-		    (save-excursion
-		      (when (re-search-backward
-			     (concat
-			      "\\("
-			      (mapconcat #'identity
-					 ebdb-snarf-name-re "\\|")
-			      "\\)")
-			     (save-excursion
-			       (if block
-				   (progn (forward-line -1)
-					  (line-beginning-position))
-				 (point-at-bol)))
-			     t)
-			;; If something goes wrong with the
-			;; name, don't worry about it.
-			(ignore-errors
-			  (ebdb-parse
-			   'ebdb-field-name
-			   (string-trim (match-string-no-properties 0)))))))
-	  ;; If NAME matches one of the records that are already in
-	  ;; BUNDLES, then assume we should be working with that record.
-	  (dolist (b bundles)
-	    (when (and (aref b 0)
-		       (string-match-p (ebdb-string name)
-				       (ebdb-string (aref b 0))))
-	      (setq bundle b))))
+	(let* ((bound (match-end 0))
+	       (name (save-excursion
+		       (when (re-search-backward
+			      name-re
+			      (if (bolp)
+				  (line-beginning-position 0)
+				(point-at-bol))
+			      t)
+			 ;; If something goes wrong with the
+			 ;; name, don't worry about it.
+			 (ignore-errors
+			   (ebdb-parse
+			    'ebdb-field-name
+			    (string-trim (match-string-no-properties 0)))))))
+	       (bundle (or (and
+			    name
+			    ;; If NAME matches one of the records that
+			    ;; are already in BUNDLES, then assume we
+			    ;; should be working with that record.
+			    (catch 'match
+			      (dolist (b bundles)
+				(when (and (aref b 0)
+					   (ebdb-record-search
+					    (aref b 0)
+					    'ebdb-field-name
+					    (ebdb-string name)))
+				  (throw 'match b)))))
+			   (make-vector 3 nil))))
 
-	(unless bundle
-	  (setq bundle (make-vector 3 nil))
-	  (when name
-	    (push name (aref bundle 1))))
+	  (when (and name (null (aref bundle 0)))
+	    (push name (aref bundle 1)))
 
-	(dolist (class ebdb-snarf-routines)
-	  (dolist (re (cdr class))
-	    (while (re-search-forward re (if block
-					     (save-excursion
-					       (forward-line)
-					       (line-end-position))
-					   (point-at-eol))
-				      t)
-	      (condition-case nil
-		  (push (ebdb-parse
-			 (car class)
-			 (match-string-no-properties 1))
-			(aref bundle 2))
+	  ;; Now find out exactly what matched, and make a field.
+	  (dolist (class ebdb-snarf-routines)
+	    (dolist (re (cdr class))
+	      (save-excursion
+		(while (re-search-forward re bound t)
+		  (condition-case nil
+		      (progn
+			;; Discard field if it's been found already.
+			(setq field (ebdb-parse (car class)
+						(match-string-no-properties 1)))
+			(unless (member field seen-fields)
+			  (push field (aref bundle 2))
+			  (push field seen-fields)))
 
-		;; If a regular expression matches but the result is
-		;; unparseable, that means the regexp is bad and should be
-		;; changed.  Later, report these errors if `ebdb-debug' is
-		;; true.
-		(ebdb-unparseable nil)))))
-	(when bundle
-	  (push bundle bundles)
-	  (setq bundle nil))
-	(when block
-	  (beginning-of-line 2))))
+		    ;; If a regular expression matches but the result is
+		    ;; unparseable, that means the regexp is bad and should be
+		    ;; changed.  Later, report these errors if `ebdb-debug' is
+		    ;; true.
+		    (ebdb-unparseable nil))))))
+	  (when (or (aref bundle 0) (aref bundle 1) (aref bundle 2))
+	    (push bundle bundles))
+	  (goto-char bound))))
     bundles))
 
 (defun ebdb-snarf-collapse (input)
   "Process INPUT, which is a list of bundled field information.
-
 INPUT is probably produced by `ebdb-snarf-collect'.  It should be
 a list of vectors, each with three elements: a single record, a
 list of name field instances, and a list of other field
@@ -251,7 +251,7 @@ existing records that match information in the bundle.  Discard
 redundant fields, or fields that are incompatible with the record
 they're grouped with.  Return the same list of (possibly altered)
 vectors, usually to `ebdb-snarf-query'."
-  (let (output rec slot-val)
+  (let (output rec)
     (pcase-dolist (`[,record ,names ,fields] input)
       (let (out-fields out-names)
 	(unless record
@@ -265,37 +265,36 @@ vectors, usually to `ebdb-snarf-query'."
 			     (append fields names)))))
 	    (setq record rec)))
 	(if record
-	    (let (slot)
+	    (progn
+	      ;; If there's a record, make sure the record can accept
+	      ;; the fields and names, and doesn't already have them.
 	      (dolist (f fields)
 		(condition-case nil
 		    (progn
-		      (setq slot (car (ebdb-record-field-slot-query
-				       (eieio-object-class record)
-				       `(nil . ,(eieio-object-class f)))))
-		      ;; Make sure that record can accept field, and doesn't
-		      ;; already have it.
-		      (unless
-			  (when (setq slot-val (ignore-errors
-						 (ebdb-record-field record slot)))
-			    (member (ebdb-string f)
-				    (mapcar #'ebdb-string
-					    (if (listp slot-val)
-						slot-val
-					      (list slot-val)))))
+		      (when (and (car-safe (ebdb-record-field-slot-query
+					    (eieio-object-class record)
+					    `(nil . ,(eieio-object-class f))))
+				 (null (ebdb-record-search
+					record
+					(eieio-object-class f)
+					(ebdb-string f))))
 			(push f out-fields)))
 		  (ebdb-unacceptable-field nil)))
 	      (dolist (name names)
 		(unless (ebdb-record-search
 			 record 'ebdb-field-name (ebdb-string name))
 		  (push name out-names))))
+	  ;; If no record, dump all the fields and names into the
+	  ;; query process.
 	  (setq out-names names
 		out-fields fields))
 	(push (vector record out-names out-fields) output)))
     output))
 
 (defun ebdb-snarf-query (input)
-  "Query the user about INPUT, which is a list of vectors of
-  bundled information representing records.
+  "Query the user about handling INPUT.
+INPUT is a list of vectors of bundled information representing
+records.
 
 Ask about field instances that we haven't been able to handle
 automatically."
